@@ -1,7 +1,7 @@
 /*
     This file is part of the KDE project.
 
-    SPDX-FileCopyrightText: 2017 Stefano Crocco <posta@stefanocrocco.it>
+    SPDX-FileCopyrightText: 2017 Stefano Crocco <stefano.crocco@alice.it>
 
     SPDX-License-Identifier: LGPL-2.1-or-later
 */
@@ -10,6 +10,9 @@
 
 #include "webenginepage.h"
 #include <webenginepart_debug.h>
+#include "webenginepartcontrols.h"
+#include "navigationrecorder.h"
+#include "choosepagesaveformatdlg.h"
 
 #include <QWebEngineView>
 #include <QWebEngineProfile>
@@ -19,10 +22,10 @@
 #include <QMimeDatabase>
 #include <QMimeType>
 #include <QTimer>
+#include <QDialog>
 
 #include <KLocalizedString>
 #include <KNotificationJobUiDelegate>
-#include <KParts/BrowserOpenOrSaveQuestion>
 #include <KJobTrackerInterface>
 #include <KIO/JobTracker>
 #include <KIO/OpenUrlJob>
@@ -39,6 +42,29 @@ WebEnginePartDownloadManager::~WebEnginePartDownloadManager()
 {
 }
 
+void WebEnginePartDownloadManager::setForceDownload(const QUrl& url, WebEnginePage *page)
+{
+    m_forcedDownloads.insert(url, page);
+}
+
+bool WebEnginePartDownloadManager::checkForceDownload(const QUrl& url, WebEnginePage *page)
+{
+    if (!page) {
+        return false;
+    }
+    bool force = m_forcedDownloads.remove(url, page) != 0;
+
+    // Bookkeeping: remove any entry with an invalid page. Note that in most cases, m_forcedDownloads
+    // will be empty, since entries are usually added just before this is called indirectly from
+    // WebEnginePage::download. The line before this removes one entry from m_forcedDownloads, which
+    // usually will be the one just added, meaning m_forcedDownloads will be empty.
+    for (const QUrl &k : m_forcedDownloads.uniqueKeys()) {
+        m_forcedDownloads.remove(k, nullptr);
+    }
+
+    return force;
+}
+
 void WebEnginePartDownloadManager::addPage(WebEnginePage* page)
 {
     if (!m_pages.contains(page)) {
@@ -54,45 +80,59 @@ void WebEnginePartDownloadManager::removePage(QObject* page)
 
 void WebEnginePartDownloadManager::performDownload(QWebEngineDownloadItem* it)
 {
+    QUrl url = it->url();
     WebEnginePage *page = qobject_cast<WebEnginePage*>(it->page());
+    if (it->isSavePageDownload()) {
+        saveHtmlPage(it, page);
+        return;
+    }
     bool forceNew = false;
     //According to the documentation, QWebEngineDownloadItem::page() can return nullptr "if the download was not triggered by content in a page"
     if (!page && !m_pages.isEmpty()) {
-        qCDebug(WEBENGINEPART_LOG) << "downloading" << it->url() << "in new window or tab";
+        qCDebug(WEBENGINEPART_LOG) << "downloading" << url << "in new window or tab";
         page = m_pages.first();
         forceNew = true;
     } else if (!page) {
-        qCDebug(WEBENGINEPART_LOG) << "Couldn't find a part wanting to download" << it->url();
+        qCDebug(WEBENGINEPART_LOG) << "Couldn't find a part wanting to download" << url;
         return;
     }
-    if (it->url().scheme() != "blob") {
-        page->download(it, forceNew);
+
+    if (WebEnginePartControls::self()->navigationRecorder()->isPostRequest(it->url(), page)) {
+        WebEnginePartControls::self()->navigationRecorder()->recordNavigationFinished(page, url);
+        downloadFile(it, KParts::BrowserOpenOrSaveQuestion::InlineDisposition, true);
+        return;
+    }
+
+    bool forceDownload = checkForceDownload(url, page);
+    if (!forceDownload && it->url().scheme() != "blob") {
+        page->downloadItem(it, forceNew);
     } else {
-        downloadBlob(it);
+        downloadFile(it, KParts::BrowserOpenOrSaveQuestion::AttachmentDisposition);
     }
 }
 
-void WebEnginePartDownloadManager::downloadBlob(QWebEngineDownloadItem* it)
+void WebEnginePartDownloadManager::downloadFile(QWebEngineDownloadItem* it, KParts::BrowserOpenOrSaveQuestion::AskEmbedOrSaveFlags disposition, bool forceNewTab)
 {
     WebEnginePage *p = qobject_cast<WebEnginePage*>(it->page());
     QWidget *w = p ? p->view() : nullptr;
     KParts::BrowserOpenOrSaveQuestion askDlg(w, it->url(), it->mimeType());
-    KParts::BrowserOpenOrSaveQuestion::Result ans = askDlg.askEmbedOrSave(KParts::BrowserOpenOrSaveQuestion::AttachmentDisposition);
+    KParts::BrowserOpenOrSaveQuestion::Result ans = askDlg.askEmbedOrSave(disposition);
     switch (ans) { 
         case KParts::BrowserOpenOrSaveQuestion::Cancel:
             it->cancel();
             return;
         case KParts::BrowserOpenOrSaveQuestion::Save:
-            saveBlob(it);
+            saveFile(it);
             break;
         case KParts::BrowserOpenOrSaveQuestion::Embed:
         case KParts::BrowserOpenOrSaveQuestion::Open:
-            openBlob(it, p);
+            openFile(it, p, forceNewTab);
             break;
     }
 }
 
-void WebEnginePartDownloadManager::saveBlob(QWebEngineDownloadItem* it)
+
+void WebEnginePartDownloadManager::saveFile(QWebEngineDownloadItem* it)
 {
     QWidget *w = it->page() ? it->page()->view() : nullptr;
     QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
@@ -101,51 +141,29 @@ void WebEnginePartDownloadManager::saveBlob(QWebEngineDownloadItem* it)
     QFileDialog dlg(w, QString(), downloadDir);
     dlg.setAcceptMode(QFileDialog::AcceptSave);
     dlg.setMimeTypeFilters(QStringList{type.name(), "application/octet-stream"});
-    dlg.setSupportedSchemes(QStringList{"file"});
     QDialog::DialogCode exitCode = static_cast<QDialog::DialogCode>(dlg.exec());
     if (exitCode == QDialog::Rejected) {
         it->cancel();
         return;
     }
     QString file = dlg.selectedFiles().at(0);
-    QFileInfo info(file);
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-    it->setPath(info.filePath());
-#else
-    it->setDownloadDirectory(info.path());
-    it->setDownloadFileName(info.fileName());
-#endif
-    it->accept();
-    it->pause();
-    WebEngineBlobDownloadJob *j = new WebEngineBlobDownloadJob(it, this);
-    KJobTrackerInterface *t = KIO::getJobTracker();
-    if (t) {
-        t->registerJob(j);
-    }
-    j->start();
+    startDownloadJob(file, it);
 }
 
-void WebEnginePartDownloadManager::openBlob(QWebEngineDownloadItem* it, WebEnginePage *page)
+void WebEnginePartDownloadManager::openFile(QWebEngineDownloadItem* it, WebEnginePage *page, bool forceNewTab)
 {
     QMimeDatabase db;
     QMimeType type = db.mimeTypeForName(it->mimeType());
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-    QString suggestedName = it->path();
-#else
     QString suggestedName = it->suggestedFileName();
-#endif
-    QString fileName = generateBlobTempFileName(suggestedName, type.preferredSuffix());
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-    it->setPath(m_tempDownloadDir.filePath(fileName));
-#else
+    QString fileName = generateDownloadTempFileName(suggestedName, type.preferredSuffix());
     it->setDownloadDirectory(m_tempDownloadDir.path());
     it->setDownloadFileName(fileName);
-#endif
-    connect(it, &QWebEngineDownloadItem::finished, this, [this, it, page](){blobDownloadedToFile(it, page);});
+    connect(it, &QWebEngineDownloadItem::finished, this, [this, it, page, forceNewTab](){
+        downloadToFileCompleted(it, page, forceNewTab);});
     it->accept();
 }
 
-QString WebEnginePartDownloadManager::generateBlobTempFileName(const QString& suggestedName, const QString& ext) const
+QString WebEnginePartDownloadManager::generateDownloadTempFileName(const QString& suggestedName, const QString& ext) const
 {
     QString baseName(suggestedName);
     if (baseName.isEmpty()) {
@@ -162,45 +180,96 @@ QString WebEnginePartDownloadManager::generateBlobTempFileName(const QString& su
 }
 
 
-void WebEnginePartDownloadManager::blobDownloadedToFile(QWebEngineDownloadItem *it, WebEnginePage *page)
+void WebEnginePartDownloadManager::downloadToFileCompleted(QWebEngineDownloadItem *it, WebEnginePage *page, bool forceNewTab)
 {
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-    QString file = it->path();
-#else
     QString file = QDir(it->downloadDirectory()).filePath(it->downloadFileName());
-#endif
     if (page) {
-        page->requestOpenFileAsTemporary(QUrl::fromLocalFile(file), it->mimeType());
+        page->requestOpenFileAsTemporary(QUrl::fromLocalFile(file), it->mimeType(), false, true);
     } else {
         KIO::OpenUrlJob *j = new KIO::OpenUrlJob(QUrl::fromLocalFile(file), it->mimeType(), this);
-        QWidget *w = page->view();
-        j->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, w ? w->window() : nullptr));
+        j->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr));
         j->start();
     }
 }
 
-WebEngineBlobDownloadJob::WebEngineBlobDownloadJob(QWebEngineDownloadItem* it, QObject* parent) : KJob(parent), m_downloadItem(it)
+void WebEnginePartDownloadManager::saveHtmlPage(QWebEngineDownloadItem* it, WebEnginePage *page)
+{
+    QWidget *parent = page ? page->view() : nullptr;
+
+    ChoosePageSaveFormatDlg *formatDlg = new ChoosePageSaveFormatDlg(parent);
+    if (formatDlg->exec() == QDialog::Rejected) {
+        return;
+    }
+    it->setSavePageFormat(formatDlg->choosenFormat());
+
+    QString downloadDir = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
+    QFileDialog dlg(parent, QString(), downloadDir);
+    dlg.setAcceptMode(QFileDialog::AcceptSave);
+
+    QString mimeType;
+    if (it->savePageFormat() == QWebEngineDownloadItem::SingleHtmlSaveFormat) {
+        mimeType = QStringLiteral("text/html");
+    } else if (it->savePageFormat() == QWebEngineDownloadItem::MimeHtmlSaveFormat) {
+        mimeType = QStringLiteral("application/x-mimearchive");
+    } else if (it->savePageFormat() == QWebEngineDownloadItem::CompleteHtmlSaveFormat) {
+        dlg.setFileMode(QFileDialog::Directory);
+        dlg.setOption(QFileDialog::ShowDirsOnly);
+    }
+
+    dlg.setMimeTypeFilters(QStringList{mimeType, "application/octet-stream"});
+
+    QDialog::DialogCode exitCode = static_cast<QDialog::DialogCode>(dlg.exec());
+    if (exitCode == QDialog::Rejected) {
+        it->cancel();
+        return;
+    }
+
+    QString file = dlg.selectedFiles().at(0);
+    startDownloadJob(file, it);
+}
+
+void WebEnginePartDownloadManager::startDownloadJob(const QString &file, QWebEngineDownloadItem *it)
+{
+    QFileInfo info(file);
+    if (info.isDir()) {
+        it->setDownloadDirectory(file);
+    } else {
+        it->setDownloadDirectory(info.dir().path());
+        it->setDownloadFileName(info.fileName());
+    }
+    it->accept();
+    it->pause();
+    WebEngineDownloadJob *j = new WebEngineDownloadJob(it, this);
+    KJobTrackerInterface *t = KIO::getJobTracker();
+    if (t) {
+        t->registerJob(j);
+    }
+    j->start();
+}
+
+
+WebEngineDownloadJob::WebEngineDownloadJob(QWebEngineDownloadItem* it, QObject* parent) : KJob(parent), m_downloadItem(it)
 {
     setCapabilities(KJob::Killable|KJob::Suspendable);
     setTotalAmount(KJob::Bytes, m_downloadItem->totalBytes());
-    connect(m_downloadItem, &QWebEngineDownloadItem::downloadProgress, this, &WebEngineBlobDownloadJob::downloadProgressed);
-    connect(m_downloadItem, &QWebEngineDownloadItem::finished, this, &WebEngineBlobDownloadJob::downloadFinished);
-    connect(m_downloadItem, &QWebEngineDownloadItem::stateChanged, this, &WebEngineBlobDownloadJob::stateChanged);
+    connect(m_downloadItem, &QWebEngineDownloadItem::downloadProgress, this, &WebEngineDownloadJob::downloadProgressed);
+    connect(m_downloadItem, &QWebEngineDownloadItem::finished, this, &WebEngineDownloadJob::downloadFinished);
+    connect(m_downloadItem, &QWebEngineDownloadItem::stateChanged, this, &WebEngineDownloadJob::stateChanged);
 }
 
-void WebEngineBlobDownloadJob::start()
+void WebEngineDownloadJob::start()
 {
-    QTimer::singleShot(0, this, &WebEngineBlobDownloadJob::startDownloading);
+    QTimer::singleShot(0, this, &WebEngineDownloadJob::startDownloading);
 }
 
-bool WebEngineBlobDownloadJob::doKill()
+bool WebEngineDownloadJob::doKill()
 {
     delete m_downloadItem;
     m_downloadItem = nullptr;
     return true;
 }
 
-bool WebEngineBlobDownloadJob::doResume()
+bool WebEngineDownloadJob::doResume()
 {
     if (m_downloadItem) {
         m_downloadItem->resume();
@@ -208,7 +277,7 @@ bool WebEngineBlobDownloadJob::doResume()
     return true;
 }
 
-bool WebEngineBlobDownloadJob::doSuspend()
+bool WebEngineDownloadJob::doSuspend()
 {
     if (m_downloadItem) {
         m_downloadItem->pause();
@@ -216,12 +285,12 @@ bool WebEngineBlobDownloadJob::doSuspend()
     return true;
 }
 
-void WebEngineBlobDownloadJob::downloadProgressed(quint64 received, quint64 total)
+void WebEngineDownloadJob::downloadProgressed(quint64 received, quint64 total)
 {
     setPercent(received*100.0/total);
 }
 
-void WebEngineBlobDownloadJob::stateChanged(QWebEngineDownloadItem::DownloadState state)
+void WebEngineDownloadJob::stateChanged(QWebEngineDownloadItem::DownloadState state)
 {
     if (state != QWebEngineDownloadItem::DownloadInterrupted) {
         return;
@@ -230,20 +299,16 @@ void WebEngineBlobDownloadJob::stateChanged(QWebEngineDownloadItem::DownloadStat
     setErrorText(m_downloadItem->interruptReasonString());
 }
 
-QString WebEngineBlobDownloadJob::errorString() const
+QString WebEngineDownloadJob::errorString() const
 {
     return i18n("An error occurred while saving the file: %1", errorText());
 }
 
-void WebEngineBlobDownloadJob::startDownloading()
+void WebEngineDownloadJob::startDownloading()
 {
     if (m_downloadItem) {
         m_startTime = QDateTime::currentDateTime();
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-        QString name = QFileInfo(m_downloadItem->path()).filePath();
-#else
         QString name = m_downloadItem->downloadFileName();
-#endif
         emit description(this, i18nc("Notification about downloading a file", "Downloading"),
                         QPair<QString, QString>(i18nc("Source of a file being downloaded", "Source"), m_downloadItem->url().toString()),
                         QPair<QString, QString>(i18nc("Destination of a file download", "Destination"), name));
@@ -251,18 +316,14 @@ void WebEngineBlobDownloadJob::startDownloading()
     }
 }
 
-void WebEngineBlobDownloadJob::downloadFinished()
+void WebEngineDownloadJob::downloadFinished()
 {
     emitResult();
     QDateTime now = QDateTime::currentDateTime();
     if (m_startTime.msecsTo(now) < 500) {
         if (m_downloadItem && m_downloadItem->page()) {
             WebEnginePage *page = qobject_cast<WebEnginePage*>(m_downloadItem->page());
-#ifdef WEBENGINEDOWNLOADITEM_USE_PATH
-            QString filePath = m_downloadItem->path();
-#else
             QString filePath = QDir(m_downloadItem->downloadDirectory()).filePath(m_downloadItem->downloadFileName());
-#endif
             emit page->setStatusBarText(i18nc("Finished saving BLOB URL", "Finished saving %1 as %2", m_downloadItem->url().toString(), filePath));
         }
     }
